/*jslint undef: true, widget: true, browser: true, onevar: true, eqeqeq: true */
/*global YAHOO,
         iw_xhr,
         iw_lib,
         iw_js_data,
         iw_transferlist_dropdown_handler,
         iw_reseller_info_handler
         confirmSubmitCheckbox,
 */
var iw_reseller_info,

iw_reseller_info_handler = {
  responder: 'Transferlist',

  /**
   * Handle XHR request success.
   */
  handle201: function( r ) {
    var result = r.getReplyData();

    /** The Global **/
    iw_reseller_info = result;
    this.populate();

    /** DIRTY HACK FOR SYNC **/
    iw_transferlist_dropdown_handler.init();
  },

  /**
   * Initialize iw_reseller_info object
   */
  init: function() {
    iw_xhr.getReply( this );
  },

  /**
   * Populate import transfer rows based on the reseller info which was grabbed
   * via XHR and placed into an object level variable.
   */
  populate: function() {
    var nickname, reseller_id, reseller, reseller_info;

    for( reseller in iw_reseller_info ) {
      if( iw_reseller_info.hasOwnProperty( reseller ) ) {
        reseller_id = reseller;
        break;
      }
    }//end for

    reseller_info = this.getResellerInfo( reseller_id );

    this.populateRowsByClass( 'iw-control-rows', reseller_info );
    this.populateRowsByClass( 'iw-data-rows', reseller_info );
  },

  /**
   * Populates the table rows with the reseller information based on the row's
   * DOM class
   */
  populateRowsByClass: function( tr_class, reseller_info ) {
    var trs = YAHOO.util.Dom.getElementsByClassName( tr_class, 'tr' );
    this.populateRows( trs, reseller_info );
  },

  /**
   * Populates all rows passed in with the reseller information.
   */
  populateRows: function( trs, reseller_info ) {
    var tr_length = trs.length,
        i;

    for( i = 0; i < tr_length; i++ ) {
      this.populateRow( trs[i], reseller_info );
    }
  },

  /**
   * Populates given row with reseller information
   */
  populateRow: function( tr, reseller_info ) {
    var selects = tr.getElementsByTagName( 'select' ),
    checkbox = tr.getElementsByTagName( 'input' )[ 0 ],
    noips = [],
    count = 0,
    k;

    // resellers
    if( selects[0] ) {
      selects[0].disabled = false;
      this.populateSelect( selects[0], reseller_info.accounts );
    }

    // db servers
    if( selects[1] ) {
      selects[1].disabled = false;
      this.populateSelect( selects[1], reseller_info.db_servers );
    }
    // ip addresses
    if( selects[2] ) {
      selects[2].disabled = false;

      // we need to see if there is more than 0 ips, and this is the only
      // way to do it since it's an object and not an actual array :(
      for( k in reseller_info.ips ) {
        if( reseller_info.ips.hasOwnProperty( k ) ) {
          count++;
          break;
        }
      }

      // some ips, populate away
      if( count > 0 ) {
        this.populateSelect( selects[2], reseller_info.ips );
        checkbox.disabled = false;
      } else {
        // no ips, say so and disable the row
        selects[2].disabled = true;
        noips.push( 'No Shared IPs Available' );
        this.populateSelect( selects[2], noips );
        checkbox.disabled = true;
      }
    }//end if selects[2] (ip addresses)

    // import type
    if( selects[3] ) {
      selects[3].disabled = false;
    }
  },

  /**
   * When given a reseller_id, a reseller_info array is built which contains
   * all reseller options, all servers the given reseller has access to, and
   * all IP addresses they have access to.
   */
  getResellerInfo: function( reseller_id ) {
    var reseller,
        nickname,
        reseller_info_array = [],
        /*
         * Private function, grabs all IP addresses the given reseller ID has
         * access to.
         */
        getIps = function( reseller_id ) {
          var ipaddr,
              reseller_info_ips = [];

          for( ipaddr in iw_reseller_info[reseller_id].ips ) {
            if( iw_reseller_info[reseller_id].ips.hasOwnProperty( ipaddr ) ) {
              reseller_info_ips[ipaddr] = iw_reseller_info[reseller_id].ips[ipaddr];
            }
          }
          return reseller_info_ips;
        },
        /*
         * Private function, grabs all database servers the given reseller has
         * access to.
         */
        getDbServers = function( reseller_id ) {
          var dbserver,
              reseller_info_servers = [];

          for( dbserver in iw_reseller_info[reseller_id].dbservers ) {
            if( iw_reseller_info[reseller_id].dbservers.hasOwnProperty( dbserver ) ) {
              reseller_info_servers[dbserver] = iw_reseller_info[reseller_id].dbservers[dbserver];
            }
          }
          return reseller_info_servers;
        };

    // Because the reseller data always has to be populated with all resellers.
    reseller_info_array.reseller_id = reseller_id;
    reseller_info_array.accounts    = [];
    reseller_info_array.ips         = [];
    reseller_info_array.db_servers  = [];

    for( reseller in iw_reseller_info ) {
      if( iw_reseller_info.hasOwnProperty( reseller ) ) {
        nickname = iw_reseller_info[reseller].nickname;

        reseller_info_array.accounts[reseller] = nickname;
      }
    }

    reseller_info_array.ips        = getIps( reseller_id );
    reseller_info_array.db_servers = getDbServers( reseller_id );

    return reseller_info_array;
  },

  /**
   * Populates the select box given the assoicative array of options.
   */
  populateSelect: function( select, options ) {
    var i = 0,
        found_current_value = false,
        current_value,
        is_selected,
        option,
        extra_option,
        /**
         * Lamda function to find the create option if needed.
         *
         * I think it's a hack that this function is here, but I can't
         * think of a better way to do this. -jmd
         */
        getImportIfExists = function( select ) {
          var options = select.options,
              option,
              opt_cnt = options.length,
              i = 0;

          if( ( select.id.match( /reseller_group\[(.+)\]/ ) !== null ) ||
              ( select.id.match( /reseller\[(.+)\]/ ) !== null ) ) {
/*
            for( option in options ) {
              if( options[option] && options[option].value === "create" ) {
                return options[option];
              }
            }
*/

            for( i = 0; i < opt_cnt; i++ ) {
              if( options[ i ] && options[ i ].value === "create" ) {
                return options[ i ];
              }
            }//end for
          }//end if

          return null;
        };

    extra_option  = getImportIfExists( select );
    current_value = select.value;

    select.options.length = 0;
    select.options.length = 0; // webkit developer tools bug workaround -pko */

    if( extra_option !== null ) {
      if( extra_option.value === current_value ) {
        found_current_value = true;
      }
      select.options.add( new Option( extra_option.text, extra_option.value ), i );
      i++;
    }//end if
    for( option in options ) {
      if( options.hasOwnProperty( option ) ) {
        if( option === current_value ) {
          found_current_value = true;
        }

        select.options.add( new Option( options[option], option ), i );
        i++;
      }//end if
    }

    if( found_current_value === true && current_value ) {
      select.value = current_value;
    } else {
      select.value = select.options[0].value;
    }
  }//end function populateSelect
},

/**
 * iw_reseller_perms_updater object
 */
iw_reseller_perms_updater = {
  /**
   * Given the value of the select (the reseller id), populate the rest of the
   * row with the approprate data.
   */
  updateInfoForReseller: function( select ) {
    var reseller_id = select.value,
        reseller_info,
        first_reseller_id,
        tr = YAHOO.util.Dom.getAncestorByTagName( select, 'tr' ),
        validateResellerId = function( reseller_id ) {
          var reseller;
          /**
           * If the reseller_id is "create" ( Importing the reseller from the other
           * server ), then we give this "imported" reseller the permissions of the
           * first reseller on the list, because they will be the one whith the
           * highest access level.
           */
          if( reseller_id === "create" ) {
            for( reseller in iw_reseller_info ) {
              if( iw_reseller_info.hasOwnProperty( reseller ) ) {
                if( iw_reseller_info[reseller].reseller_id ) {
                  reseller_id = iw_reseller_info[reseller].reseller_id;
                  break;
                }
              }
            }//end for
          }//end if( reseller_id === "create" )
          return reseller_id;
        },
        valid_reseller_id = validateResellerId( reseller_id );

    reseller_info = iw_reseller_info_handler.getResellerInfo( valid_reseller_id );
    iw_reseller_info_handler.populateRow( tr, reseller_info );
  }
},

/**
 * iw_transferlist_dropdown_handler object
 */
iw_transferlist_dropdown_handler = {
  /**
   * When the master checkbox is checked, and then a row under it has been
   * unchecked, we are going to want to uncheck the master row, as it is no
   * longer in a valid state.
   */
  toggleMasterCheckbox: function( e ) {
    var getMasterCheck = function( element ) {
      var yay_me = YAHOO.util.Event.getTarget( e ),
          my_group = yay_me.attributes.checkbox_group.value,
          checkbox_group_master;

      if( !YAHOO.lang.isUndefined( element.attributes.checkbox_group_master ) ) {
        checkbox_group_master = element.attributes.checkbox_group_master.value;
        if( checkbox_group_master === my_group ) {
          return true;
        }
      }
      return false;
    },
    master_check = YAHOO.util.Dom.getElementBy( getMasterCheck, 'input' );

    master_check.checked = false;
  },

  /**
   * When the master checkbox is checked, we want to check all checkboxes in
   * it's group.
   */
  toggleAllCheckboxes : function( e ) {
    var getChecks = function( element ) {
      var master_check, cgm_value, checkbox_group;

      master_check = YAHOO.util.Event.getTarget( e );
      if( ( element.type === 'checkbox' ) && ( element.disabled === false ) ) {
        cgm_value = master_check.attributes.checkbox_group_master.value;

        if( element.attributes.checkbox_group ) {
          checkbox_group = element.attributes.checkbox_group.value;
        }

        return cgm_value === checkbox_group;
      }//end if
      return false;
    },//end function
    checks = YAHOO.util.Dom.getElementsBy( getChecks, 'input' ),
    checks_length = checks.length,
    i;

    for( i = 0; i < checks_length; i++ ) {
      if( checks[i] ) {
        checks[i].checked = this.checked;
      }//end if
    }//end for
  },//end function

  /**
   * Toggles all Select boxes based on the select box in the "advanced options"
   * row.
   */
  toggleAllSelects : function( e ) {
    iw_transferlist_dropdown_handler.manualToggleSelects( this );
  },

  /**
   * Toggles all Select boxes based on the select box in the "advanced options"
   * row.
   */
  manualToggleSelects: function( select ) {
    var tr = YAHOO.util.Dom.getAncestorByTagName( select, 'tr' ),
        cell_index = -1,
        i,
        selects,
        selects_length,
        checkSelectForOption = function( select, value ) {
          var options_length = select.options.length,
              i;

          for( i = 0; i < options_length; i++ ) {
            if( select.options[i] && select.options[i].value === value ) {
              return true;
            }
          }//end for
        }; //end function

    selects        = tr.getElementsByTagName( 'select' );
    selects_length = selects.length;
    for( i = 0; i < selects_length; i++ ) {
      if( selects[i].id.match( /reseller_group\[(.+)\]/ )  !== null ) {
        iw_reseller_perms_updater.updateInfoForReseller( selects[i] );
      }
      if( selects[i].id === select.id ) {
        cell_index = i;
      }
    }//end for

    if( cell_index === -1 ) {
      return;
    }

    tr = YAHOO.util.Dom.getNextSibling( tr );
    while( YAHOO.lang.isNull( tr ) === false ) {
      /*
       * Kind of a hack, but if we've hit a seperator row then we know we've
       * changed all the dropdowns for that particular reseller.
       */
      if( YAHOO.util.Dom.hasClass( tr, 'iw-seperator-rows' ) === true ) {
        break;
      }

      selects = tr.getElementsByTagName( 'select' );
      if( YAHOO.lang.isUndefined( selects[cell_index] ) === false ) {

        if( checkSelectForOption( selects[cell_index], select.value ) === true ) {
          selects[cell_index].value = select.value;
        }
        /*
         * If this select box is the reseller select box then we want to fire
         * the execute event on the row handler to update all the related
         * select boxes.
         */
        if( selects[cell_index].id.match( /reseller\[(.+)\]/ ) !== null ) {
          iw_reseller_perms_updater.updateInfoForReseller( selects[cell_index] );
        }//end if
      }//end if

      tr = YAHOO.util.Dom.getNextSibling( tr );
    }//end while
  },

  /**
   * Inits the object, but initialization is done when iw_reseller_info_handler
   * has finished it's XHR request.
   */
  init: function() {
    var tr = YAHOO.util.Dom.getElementsByClassName( 'iw-control-rows', 'tr' ),
        tr_length = tr.length,
        selects,
        inputs,
        mapped_id,
        i,
        updateResellerInfo = function() {
          iw_reseller_perms_updater.updateInfoForReseller( this );
        };

    for( i = 0; i < tr_length; i++ ) {
      selects = tr[i].getElementsByTagName( 'select' );
      inputs  = tr[i].getElementsByTagName( 'input' );

      // Puts the event handler on the first checkbox
      if( inputs[0] ) {
        if( inputs[0].id.match( /checkbox_group\[(.+)\]/ ) !== null ) {
          YAHOO.util.Event.on( inputs[0], 'click', this.toggleAllCheckboxes );
        }
        /*
         * This is what sets the approprate reseller item in the dropdown
         */
        if( selects[0].id.match( /reseller_group\[(.+)\]/ ) !== null ) {
          mapped_id        = selects[0].attributes.mapped_id.value;
          selects[0].value = mapped_id;

          // removes the "bad" attribute, as we no longer need it.
          selects[0].removeAttribute( 'mapped_id' );
          iw_reseller_perms_updater.updateInfoForReseller( selects[0] );
          this.manualToggleSelects( selects[0] );
        }
      }
      YAHOO.util.Event.on( selects, 'change', this.toggleAllSelects );
    }//end for

    tr        = YAHOO.util.Dom.getElementsByClassName( 'iw-data-rows', 'tr' );
    tr_length = tr.length;
    for( i = 0; i < tr_length; i++ ) {
      selects = tr[i].getElementsByTagName( 'select' );
      inputs  = tr[i].getElementsByTagName( 'input' );

      if( inputs[0] ) {
        YAHOO.util.Event.on( inputs[0], 'click', this.toggleMasterCheckbox );
      }

      if( selects[0] ) {
        if( selects[0].id.match( /reseller\[(.+)\]/ ) !== null ) {
          YAHOO.util.Event.on( selects[0], 'change', updateResellerInfo );
        }
      }
    }//end for
  }//end function init
};

iw_reseller_info_handler.init();

YAHOO.util.Event.on( 'import', 'change',  function( e ) {
  var select, current_ip, i,
      Dom = YAHOO.util.Dom,
      ips_in_use = {},
      ip_selects = Dom.getElementsByClassName( 'ipaddr', 'select' ),
      boxes = document.getElementsByName( 'input[users][]' );

  for( i in boxes ) {
    if( boxes.hasOwnProperty( i ) ) {
      if( boxes[i].checked === true ) {
        select     = document.getElementById( 'ipaddr[' + boxes[i].id + ']' );
        current_ip = select.value;

        if( current_ip in ips_in_use ) {
          e.target.selectedIndex = 0;
          alert( '##LG_CANNOT_SPECIFY_DEDICATED_IP_TWICE##' );
          return false;
        }
        if( iw_js_data.ips[current_ip].type === 'static' ) {
          ips_in_use[current_ip] = current_ip;
        }
      }
    }
  }

  confirmSubmitCheckbox( e.target,
                         "input[users][]",
                         "##LG_SELECTED_ACCOUNTS##",
                         "##LG_CONFIRM_ACTION##",
                         "##LG_NOTHING_SELECTED##" );
});